﻿Imports System.Math
Imports System.Drawing.Drawing2D

Public Class Form1
    ' The circles.
    Private Centers As New List(Of PointF)()
    Private Radii As New List(Of Single)()

    ' The index of a new circle we are drawing.
    Private NewCircle As Integer = -1

    ' Remove all circles.
    Private Sub btnClear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClear.Click
        Centers = New List(Of PointF)()
        Radii = New List(Of Single)()
        Me.Invalidate()
    End Sub

    ' Start drawing a new circle.
    Private Sub Form1_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        Centers.Add(e.Location)
        Radii.Add(0)
        NewCircle = Centers.Count - 1
    End Sub

    ' Update the new circle.
    Private Sub Form1_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If (NewCircle < 0) Then Return
        Dim dx As Single = e.X - Centers(NewCircle).X
        Dim dy As Single = e.Y - Centers(NewCircle).Y
        Radii(NewCircle) = CSng(Sqrt(dx * dx + dy * dy))
        Me.Invalidate()
    End Sub

    ' Finish drawing a new circle.
    Private Sub Form1_MouseUp(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseUp
        ' If the radius is 0, remove the new circle.
        If (Radii(NewCircle) < 1) Then
            Centers.RemoveAt(NewCircle)
            Radii.RemoveAt(NewCircle)
        End If

        NewCircle = -1
        Me.Invalidate()
    End Sub

    ' Draw the circles.
    Private Sub Form1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Me.Paint
        e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias

        ' See if we are currently drawing a new circle.
        If (NewCircle < 0) Then
            ' We are not drawing a new circle.
            ' Draw the regions.
            ' Find the RegionInfos for the circles.
            Dim regioninfos As New List(Of RegionInfo)()
            For i As Integer = 0 To Centers.Count - 1
                ' Make a RegionInfo for this circle.
                Dim new_regioninfo As New RegionInfo(1, Centers(i), Radii(i))

                ' Make intersections between the new RegionInfo
                ' and other RegionInfos already on the list.
                new_regioninfo.MakeIntersections(e.Graphics, regioninfos)

                ' Add the new RegionInfo to the list.
                regioninfos.Add(new_regioninfo)
            Next i

            ' Find the largest RegionInfo.Count.
            Dim max_count As Integer = 0
            For Each region_info As RegionInfo In regioninfos
                If (max_count < region_info.Count) Then
                    max_count = region_info.Count
                End If
            Next region_info

            ' Draw the RegionInfos.
            For Each region_info As RegionInfo In regioninfos
                region_info.Draw(e.Graphics, Me.Font, max_count)
            Next region_info
        End If

        ' Draw the circles.
        For i As Integer = 0 To Centers.Count - 1
            e.Graphics.DrawEllipse(Pens.Blue, _
                Centers(i).X - Radii(i), Centers(i).Y - Radii(i), _
                2 * Radii(i), 2 * Radii(i))
        Next i
    End Sub

    ' Find the intersection of all of the circles.
    Private Function FindCircleIntersections(ByVal centers As List(Of PointF), ByVal radii As List(Of Single)) As Region
        If (centers.Count < 1) Then Return Nothing

        ' Make a region.
        Dim result_region As New Region()

        ' Intersect the region with the circles.
        For i As Integer = 0 To centers.Count - 1
            Using circle_path As New GraphicsPath()
                circle_path.AddEllipse( _
                    centers(i).X - radii(i), centers(i).Y - radii(i), _
                    2 * radii(i), 2 * radii(i))
                result_region.Intersect(circle_path)
            End Using
        Next i

        Return result_region
    End Function

    ' Return the centroid of the region.
    Private Function RegionCentroid(ByVal region As Region, ByVal transform As Matrix) As PointF
        Dim cx As Single = 0
        Dim cy As Single = 0
        Dim total_weight As Single = 0
        For Each rect As RectangleF In region.GetRegionScans(transform)
            Dim rect_weight As Single = rect.Width * rect.Height
            cx += rect_weight * (rect.Left + rect.Width / 2.0F)
            cy += rect_weight * (rect.Top + rect.Height / 2.0F)
            total_weight += rect_weight
        Next rect

        Return New PointF(cx / total_weight, cy / total_weight)
    End Function
End Class
